#!/usr/bin/perl -w
###########################################################
#
# SELECTpro Solo
#
#  configure.pl: to configure the installation of SELECTpro Solo
#
# Author: Arlo Randall
# Date:   11-5-2008
# Institute for Genomics and Bioinformatics
# School of Information and Computer Science
# University of California, Irvine
#
##########################################################


#######Customize settings here###############################################

#set installation directory of SELECTpro to the full path of your unzipped SELECTpro directory 
# $install_dir = "/replace/this/path/selectpro_solo/";
$install_dir = "/home/baldig/ProteinStructure/SELECTpro/selectpro_solo/";

##########End of Customizaton################################################



################Don't Change the code below##############

if (! -d $install_dir)
{
	die "can't find installation directory.\n";
}
if ( substr($install_dir, length($install_dir) - 1, 1) ne "/" )
{
	$install_dir .= "/"; 
}

#check if the installation directory is right
#the configuration file must run in the installation directory
$cur_dir = `pwd`;  
chomp $cur_dir; 
$configure_file = "$cur_dir/configure.pl";
if (! -f $configure_file || $install_dir ne "$cur_dir/")
{
	die "Please check the installation directory setting and run the configure program there.\n";
}

$bin_dir = "${install_dir}bin/";
$data_dir = "${install_dir}data/";
$script_dir = "${install_dir}script/";
$test_sp_dir = "${install_dir}test/";

if ( ! -d $bin_dir || ! -d $data_dir || ! -d $script_dir || ! -d $test_sp_dir )
{
	die "some sub directories don't exist. check the installation tar ball.\n";
}

# define default rotamer library
$rotamer_library = "${install_dir}data/rotamers/rotamer_library.txt";


############################Generate Shell Scripts For user##############################

#generate script to run selectpro, given a pxml file and directory containing models to score
open(SHELL, ">${bin_dir}selectpro_score_dir.sh") || die "can't selectpro_score_dir.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score all models in a directory with selectpro\n";
print SHELL "if [ \$# -ne 3 ]\n";
print SHELL "then\n\techo \"need 3 parameters: pxml_file, models_directory, output_file\" \n\texit 1\nfi\n";
print SHELL "#output: results file with one model per line and individual energy terms\n"; 
print SHELL "${script_dir}selectpro_score_dir.pl $install_dir \$1 \$2 \$3 individual\n";
close SHELL;

#generate script to run selectpro, given a pxml file and directory containing models to score
open(SHELL, ">${bin_dir}selectpro_score_dir_sum.sh") || die "can't selectpro_score_dir_sum.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score all models in a directory with selectpro\n";
print SHELL "if [ \$# -ne 3 ]\n";
print SHELL "then\n\techo \"need 3 parameters: pxml_file, models_directory, output_file\" \n\texit 1\nfi\n";
print SHELL "#output: results file with one model per line and sum of energy terms\n"; 
print SHELL "${script_dir}selectpro_score_dir.pl $install_dir \$1 \$2 \$3 sum\n";
close SHELL;

#generate script to run selectpro, given a pxml file and list of models
open(SHELL, ">${bin_dir}selectpro_score_list.sh") || die "can't create selectpro_list.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score set of models in a list using selectpro\n";
print SHELL "if [ \$# -ne 3 ]\n";
print SHELL "then\n\techo \"need 3 parameters: pxml_file, models_list, output_file\" \n\texit 1\nfi\n";
print SHELL "#output: results file with one model per line and individual energy terms\n"; 
print SHELL "${script_dir}selectpro_score_list.pl $install_dir \$1 \$2 \$3 individual\n";
close SHELL; 

#generate script to run selectpro, given a pxml file and list of models
open(SHELL, ">${bin_dir}selectpro_score_list_sum.sh") || die "can't create selectpro_score_list.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score set of models in a list using selectpro\n";
print SHELL "if [ \$# -ne 3 ]\n";
print SHELL "then\n\techo \"need 3 parameters: pxml_file, models_list, output_file\" \n\texit 1\nfi\n";
print SHELL "#output: results file with one model per line and sum of energy terms\n"; 
print SHELL "${script_dir}selectpro_score_list.pl $install_dir \$1 \$2 \$3 sum\n";
close SHELL; 

#generate script to run selectpro on a single pxml file with model coordinates
open(SHELL, ">${bin_dir}selectpro_score_model.sh") || die "can't selectpro_score_model.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score single model already loaded into pxml file\n";
print SHELL "if [ \$# -ne 2 ]\n";
print SHELL "then\n\techo \"need 2 parameters: pxml_file output_file\" \n\texit 1\nfi\n";
print SHELL "#output: all energy term scores on one line in output file\n\n"; 
print SHELL "${bin_dir}selectpro \$1 $rotamer_library\n";
print SHELL "mv \$1.en \$2\n";
print SHELL "rm \$1.pdb\n";

#generate script to run selectpro on a single pxml file with model coordinates and keep model with predicted sidechains
open(SHELL, ">${bin_dir}selectpro_score_model_save_sc.sh") || die "can't selectpro_score_model_save_sc.sh.\n";
print SHELL "#!/bin/sh\n";
print SHELL "# score single model already loaded into pxml file and save model with predicted side-chains\n";
print SHELL "if [ \$# -ne 3 ]\n";
print SHELL "then\n\techo \"need 3 parameters: pxml_file scores_output_file pdb_output_file\" \n\texit 1\nfi\n";
print SHELL "#output: \n"; 
print SHELL "# 	scores_output_file: all energy term scores on one line in output file\n"; 
print SHELL "# 	pdb_output_file: .pdb format model side-chains predicted by SELECTpro\n\n"; 
print SHELL "${bin_dir}selectpro \$1 $rotamer_library\n";
print SHELL "mv \$1.en \$2\n";
print SHELL "mv \$1.pdb \$3\n";

close SHELL; 

`chmod 755 ${bin_dir}*.sh`; 
`chmod 755 ${bin_dir}selectpro`;
`chmod 755 ${script_dir}*`;

