#!/usr/bin/perl -w
#
# Description:
#	map2pxml.pl - create pxml file from predicted features
#
# Usage: map2pxml.pl input_file output_file [contact_threshold contact_type]
# 
# Default values: contact_threshold = 12
#       
############################################################

use strict;

my $input_filename = shift @ARGV;
my $output_filename = shift @ARGV;

my $contact_threshold = (@ARGV?shift @ARGV:12);

open(INPUT,"<$input_filename") or die "Couldn't open input file: $input_filename\n";
open(OUTPUT,">$output_filename") or die "Couldn't open output file: $output_filename\n";

my $temp;
my $i;
my $j;

$temp = <INPUT>;
my $protein = &trim($temp);

$temp = <INPUT>;
my $aa = &trim($temp);
my $length = length($aa);

$temp = <INPUT>;
my $ss = &trim($temp);

$temp = <INPUT>;
my $acc = &trim($temp);
chomp($acc);
$temp = $acc;
$acc =~ s/b/-/g; 

my $map_present = 0;

my @raw_cm = <INPUT>;

my @cm = ();

for $i(0..$#raw_cm) {
	if($raw_cm[$i] =~ /0|1/) {
		push(@cm,$raw_cm[$i]);
	}
}

if($#cm == ($length-1)) {
	$map_present = 1;
}	

my @half_cm = ();


my @row = ();
$temp = "";
for $i(0..$#cm) {
	$temp = $cm[$i];
	@row = split(/\s+/,$temp);
	
	$temp = "";
	for $j(0..$i) {
		$temp .= "$row[$j] ";		
	}
	$temp .= "\n";
	
	push(@half_cm,$temp);
}



print OUTPUT "<protein name = \"$protein\" length = \"$length\">\n";
print OUTPUT "<amino_acids>\n";
print OUTPUT "\t$aa\n";
print OUTPUT "</amino_acids>\n";
print OUTPUT "<secondary_structure>\n";
print OUTPUT "\t<predicted>\n";
print OUTPUT "\t\t$ss\n";
print OUTPUT "\t</predicted>\n";
print OUTPUT "</secondary_structure>\n";
print OUTPUT "<solvent_acc>\n";
print OUTPUT "\t<predicted>\n";
print OUTPUT "\t\t$acc\n";
print OUTPUT "\t</predicted>\n";
print OUTPUT "</solvent_acc>\n";

if($map_present == 1) {
	print OUTPUT "<contact_map threshold = \"$contact_threshold\" type = \"half\">\n";
	print OUTPUT "\t<predicted>\n";
	
	foreach my $line (@half_cm) {
	    print OUTPUT "\t\t$line";
	}
	print OUTPUT "\t</predicted>\n";
	print OUTPUT "</contact_map>\n";
}

print OUTPUT "</protein>\n";



#*****************************************
# trim whitespace from edges of string
sub trim {
	my $temp_string = $_[0];
	$temp_string =~ s/^\s+//;
	$temp_string =~ s/\s+$//;
	return $temp_string;
}