#!/usr/bin/perl -w
# 
# File: selectpro_score_list.pl
# Description:
#  score set of models in a list and return:
#  
# Output:
#	In output file one model is displayed per row
#	if 'scoring' parameter is set to "sum", sum of all energy terms will be returned
#	 otherwise raw scores for all terms 

if(@ARGV != 5) {
	print "usage: perl selectpro_score_list.pl install_dir pxml_file model_list output_file scoring\n";
	exit(0);
}

$install_dir = $ARGV[0];
$pxml_file = $ARGV[1];
$model_list = $ARGV[2];
$output_file = $ARGV[3];
$scoring = $ARGV[4];

$bin_dir = "${install_dir}bin/";
$script_dir = "${install_dir}script/";
$temp_dir = "${install_dir}bin/temp/";
$rotamer_file = "${install_dir}data/rotamers/rotamer_library.txt";

if (-f $pxml_file) {

	# get just pxml file name if full path is given
	if($pxml_file =~ /\//) {
		$temp = $pxml_file;
		chomp($temp);
		@row = split(/\//,$temp);
		$pxml_name = $row[$#row];
	}
	else {
		$pxml_name = $pxml_file;
	}
}
else {
	die "can't open pxml file:\n$pxml_file\n\n"; 	
}

# get sequence length from pxml file to use in model check
$seq_length = -1;
open(PXML,$pxml_file);
$pxml_text = <PXML>;
close(PXML);

$pxml_text =~ /<protein.*"(.*?)">/;
$seq_length = $1;

# read in list of models (either full path or just name is OK)
open(LIST,$model_list);
@raw_list = <LIST>;
close(LIST);

@list = ();
for $i(0..$#raw_list) {

	$temp = &trim($raw_list[$i]);
	
	if($temp =~ /\w+/) {
		push(@list,$temp);
	}	
}

# score models
$results_file = "${temp_dir}$pxml_name.results";

#$cur_dir = `pwd`;
#chomp($cur_dir);

if ( substr($cur_dir, length($cur_dir) - 1, 1) ne "/" )
{	$cur_dir .= "/"; 	}

# track incomplete models
@incomplete_models = ();
for $i(0..$#list) {

	$temp = $list[$i];

	$model_file = $temp;

	print "*******************************************************\n";
	print "PROCESSING MODEL\n";
	print $list[$i]."\n";
	
	
	#$full = 0;
	# get just pxml file name if full path is given
	if($list[$i] =~ /\//) {
		@row = split(/\//,$temp);
		$model_name = $row[$#row];
		$full_model_file = $model_file;
	}
	else {
		$model_name = $temp;
		# $full_model_file = "${cur_dir}$temp";
	}
	
	
	# if (-f $full_model_file) {
	if (-f $model_file) {
		# check model format
		$temp = `${script_dir}check_model.pl $seq_length $model_file`;
		$format = int($temp);			
	}
	else {
		$format = -2; # file does not exist			
	}	
	
	print "FORMAT $format\t$seq_length\n";
	
	# output  of check_model.pl
	#	1: correct number of backbone atoms are present
	#	0: incorrect number of backbone atoms (but in pdb format)
	#	-1: file is not in pdb format
	
	if($format == 1) {
	
		system("${script_dir}add_bbone2pxml.pl $pxml_file $model_file ${temp_dir}$model_name.pxml");
	
		# run selectpro executable
		system("${bin_dir}selectpro ${temp_dir}$model_name.pxml $rotamer_file");
		# system("${bin_dir}selectpro ${temp_dir}$model_name.pxml");
				
		$en_file = "${temp_dir}$model_name.pxml.en";
		open(ENERGIES,$en_file);
		@text = <ENERGIES>;
		close(ENERGIES);
		
		$energies= $text[0];
		chomp($energies);
			
		@row = split(/\s+/,$energies);
		$sum = 0;
		for $j(0..$#row) {
			$sum += $row[$j];
		}
			
		if($scoring eq "sum") {
			$energies = $sum;
		}
		else {
			$energies .= "\t$sum";
		}
		
		$results_line = "$model_name\t$energies\n";
		
		# add score to energy file
		open(RESULTS,">>$results_file");
		print RESULTS $results_line;
		close(RESULTS);

		# clean up .pxml and .pxml.en files in /bin/ directory
		system("rm ${temp_dir}$model_name.pxml");
		system("rm ${temp_dir}$model_name.pxml.en");
		system("rm ${temp_dir}$model_name.pxml.pdb");

	}
	else {
	
		# add incomplete message to results file
		if($format == 0) {
			$results_line = "$model_name\tINCOMPLETE_MODEL\n";		
		}
		# add non-pdb message to results file
		elsif($format == -1) {
			$results_line = "$model_name\tNON-PDB_FORMAT\n";		
		}
		# unable to open file
		elsif($format == -2) {
			$results_line = "$model_name\tUNABLE_TO_OPEN_FILE\n";
		}
		
		push(@incomplete_models,$results_line);
	}	
}	


open(RESULTS,$results_file);
@res_text = <RESULTS>;
close(RESULTS);

@results = ();
for $i(0..$#res_text) {
	$temp = $res_text[$i];
	chomp($temp);
	
	@row = split(/\s+/,$temp);
	
	if(@row > 0) {
		push(@results,[@row]);
	}
}

open(OUT,">$output_file");
	
# only display sum if selected
if($scoring eq "sum") {
	
	@sorted_results = sort { $a->[1] <=> $b->[1] } @results; 
		
	print OUT "Sum of All Energy Terms (lower scores are better)\n";
	print OUT "Model\tE_TOTAL\n";
	for $i(0..$#sorted_results) {
		$temp = sprintf ("%.1f",$sorted_results[$i][1]);
		print OUT "$sorted_results[$i][0]\t$temp\n";
	}	
}
# otherwise display individual terms
else {

	@sorted_results = sort { $a->[17] <=> $b->[17] } @results; 
	
	print OUT "Raw Scores of Individual Energy Terms (lower scores are better)\n";
	print OUT "Model\tPRED-SS_h\tPRED-SS_s\tPRED-ACC\tPRED-CM_fn\tPRED-CM_fp\tBETA\tBB-REP\t";
	print OUT "CT-REP\tSTAT-ENV\tSTAT-PW-CI\tSTAT-PW-CD\tROG\tSC-HB\tLEN-JONES\tSOLVATION\tELECTRO\n";
		
	# last row is sum, don't display
	for $i(0..$#sorted_results) {
		print OUT $sorted_results[$i][0];
		for $j(1..15) {
			$temp = sprintf ("%.1f",$sorted_results[$i][$j]);
			print OUT "\t$temp";
		}
		$temp = sprintf ("%.1f",$sorted_results[$i][16]);
		print OUT "\t$temp\n";		
	}		
}

# display incomplete / ill-formatted files
for $i(0..$#incomplete_models) {
	print OUT $incomplete_models[$i];
}

close(OUT);

system("rm $results_file");

#*****************************************
# trim whitespace from edges of string
sub trim {
	my $temp_string = $_[0];
	$temp_string =~ s/^\s+//;
	$temp_string =~ s/\s+$//;
	return $temp_string;
}
